==> em_tasks-0.1.61-py2.py3-none-any.whl: em_tasks/tools/custom_label_updater.py <==
import json
import math
import pdb

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *


class CustomLabelUpdater():
  def __init__(self):
    self.gss = dict()
    self.merchant_ids = dict()
    gmc_stores = GmcStore.select().execute()
    for gmc_store in gmc_stores:
      # if gmc_store.merchant_id == '654568556':
      #   continue

      self.merchant_ids[gmc_store.merchant_id] = gmc_store.id
      self.gss[gmc_store.id] = GoogleShopping(
        gmc_store.merchant_id, json.loads(gmc_store.service_account_data))

  def run(self):
    batch_size = 5000
    gps = dict()
    for merchant_id, gmc_store_id in self.merchant_ids.items():
      page = 1
      query = GmcProduct.select().where(GmcProduct.merchant_id == merchant_id).order_by(GmcProduct.id.asc())
      total_cnt = query.count()
      pages = int((total_cnt + batch_size) / batch_size)
      while page <= pages:
        for gp in query.paginate(page, batch_size).execute():
          gps[gp.offer_id] = gp

        self.process_products(merchant_id, gps)
        gps.clear()

        page += 1

      if gps:
        self.process_products(merchant_id, gps)
        gps.clear()

  def process_products(self, merchant_id, gps):
    custom_labels = dict()

    offer_ids = list(gps.keys())
    gaps = dict()
    for gap in GoogleAdsPerformance.select().where((GoogleAdsPerformance.merchant_id == merchant_id) & (GoogleAdsPerformance.item_id.in_(offer_ids))).execute():
      gaps[gap.item_id] = gap

    handles = [gp.handle for gp in gps.values()]
    pss = dict()
    for ps in ProductSource.select().where(ProductSource.handle.in_(handles)).dicts():
      pss[ps['handle']] = ps

    for item_id, gp in gps.items():
      custom_label0 = 'Other'
      if gp.handle in pss:
        ps = pss[gp.handle]
        custom_label0 = ps['source']

      prod_id = gp.gmc_id
      custom_label1 = 'Zombies'
      performance = gaps.get(item_id, None)
      if performance:
        if performance.clicks > 10 and performance.roas > 3:
          custom_label1 = 'Click>10_Roas>300'
        elif performance.clicks > 10 and performance.roas > 2 and performance.roas < 3:
          custom_label1 = 'Click>10_200<Roas<300'
        elif performance.clicks > 10 and performance.roas < 2:
          custom_label1 = 'Click>10_Roas<200'
        elif performance.clicks < 10 and performance.roas > 3:
          custom_label1 = 'Click<10_Roas>300'
        elif performance.clicks < 10 and performance.roas > 2 and performance.roas < 3:
          custom_label1 = 'Click<10_200<Roas<300'
        elif performance.clicks < 10 and performance.roas < 2:
          custom_label1 = 'Click<10_Roas<200'
        elif performance.impressions > 500 and performance.clicks < 1:
          custom_label1 = 'Imp>500_Click<1'
        elif performance.impressions < 100:
          custom_label1 = 'Zombies'
        else:
          custom_label1 = 'Normal'

      custom_labels[item_id] = {
        'productId': prod_id,
        'customLabel0': custom_label0,
        'customLabel1': custom_label1,
      }

    self.update_gmc_custom_label(merchant_id, custom_labels)

  def update_gmc_custom_label(self, merchant_id, custom_labels):
    if merchant_id not in self.merchant_ids:
      return

    gmc_store_id = self.merchant_ids[merchant_id]
    items = list(custom_labels.values())
    n = 250
    chunks = [items[x:x + n] for x in range(0, len(items), n)]
    for chunk in chunks:
      gs = self.gss[gmc_store_id]
      result = gs.update_products(chunk)

      logger.info(
        '[GoogleMerchantCustomLabelUpdate] Succeed: %s, Failed: %s',
        result['succeed_cnt'], result['failed_cnt'])
      logger.info(
        '[GoogleMerchantCustomLabelUpdate] %s', chunk)

==> em_tasks-0.1.65-py2.py3-none-any.whl: em_tasks/tools/custom_label_updater.py <==
import json
import math
import pdb

from shopping_utils.google_shopping import GoogleShopping

from em_tasks import logger
from em_tasks.store.models import *


class CustomLabelUpdater():
  def __init__(self):
    self.gss = dict()
    self.merchant_ids = dict()
    gmc_stores = GmcStore.select().execute()
    for gmc_store in gmc_stores:
      # if gmc_store.merchant_id == '654568556':
      #   continue

      self.merchant_ids[gmc_store.merchant_id] = gmc_store.id
      self.gss[gmc_store.id] = GoogleShopping(
        gmc_store.merchant_id, json.loads(gmc_store.service_account_data))

  def run(self):
    batch_size = 5000
    gps = dict()
    for merchant_id, gmc_store_id in self.merchant_ids.items():
      page = 1
      query = GmcProduct.select().where(GmcProduct.merchant_id == merchant_id).order_by(GmcProduct.id.asc())
      total_cnt = query.count()
      pages = int((total_cnt + batch_size) / batch_size)
      while page <= pages:
        for gp in query.paginate(page, batch_size).execute():
          gps[gp.offer_id] = gp

        self.process_products(merchant_id, gps)
        gps.clear()

        page += 1

      if gps:
        self.process_products(merchant_id, gps)
        gps.clear()

  def process_products(self, merchant_id, gps):
    custom_labels = dict()

    offer_ids = list(gps.keys())
    gaps = dict()
    for gap in GoogleAdsPerformance.select().where((GoogleAdsPerformance.merchant_id == merchant_id) & (GoogleAdsPerformance.item_id.in_(offer_ids))).execute():
      gaps[gap.item_id] = gap

    handles = [gp.handle for gp in gps.values()]
    pss = dict()
    for ps in ProductSource.select().where(ProductSource.handle.in_(handles)).dicts():
      pss[ps['handle']] = ps

    for item_id, gp in gps.items():
      custom_label0 = 'Other'
      if gp.handle in pss:
        ps = pss[gp.handle]
        custom_label0 = ps['source']

      prod_id = gp.gmc_id
      custom_label1 = 'Zombies'
      custom_label2 = 'U'
      performance = gaps.get(item_id, None)
      if performance:
        if performance.cost > 15 and performance.conversions < 1:
          custom_label1 = 'Cost>15_Con<1'
        elif performance.average_cpc > 0.3 and performance.roas < 2:
          custom_label1 = 'CPC>0.3_Roas<200'
        elif performance.cost > 10 and performance.roas < 2:
          custom_label1 = 'Cost>10_Roas<200'
        elif performance.clicks > 10 and performance.roas > 3:
          custom_label1 = 'Click>10_Roas>300'
        elif performance.clicks > 10 and performance.roas > 2 and performance.roas < 3:
          custom_label1 = 'Click>10_200<Roas<300'
        elif performance.clicks > 10 and performance.roas < 2:
          custom_label1 = 'Click>10_Roas<200'
        elif performance.clicks < 10 and performance.roas > 3:
          custom_label1 = 'Click<10_Roas>300'
        elif performance.clicks < 10 and performance.roas > 2 and performance.roas < 3:
          custom_label1 = 'Click<10_200<Roas<300'
        elif performance.clicks < 10 and performance.roas < 2:
          custom_label1 = 'Click<10_Roas<200'
        elif performance.impressions > 500 and performance.clicks < 1:
          custom_label1 = 'Imp>500_Click<1'
        elif performance.impressions < 100:
          custom_label1 = 'Zombies'
        else:
          custom_label1 = 'Normal'

        if performance.average_cpc < 0.2:
          if performance.roas < 2:
            custom_label2 = 'L_Roas<200'
          elif performance.roas < 3:
            custom_label2 = 'L_200<Roas<300'
          else:
            custom_label2 = 'L_Roas>300'
        elif performance.average_cpc < 0.3:
          if performance.roas < 2:
            custom_label2 = 'M_Roas<200'
          elif performance.roas < 3:
            custom_label2 = 'M_200<Roas<300'
          else:
            custom_label2 = 'M_Roas>300'
        elif performance.average_cpc >= 0.3:
          if performance.roas < 2:
            custom_label2 = 'H_Roas<200'
          elif performance.roas < 3:
            custom_label2 = 'H_200<Roas<300'
          else:
            custom_label2 = 'H_Roas>300'
        else:
          custom_label2 = 'U'

      custom_labels[item_id] = {
        'productId': prod_id,
        'customLabel0': custom_label0,
        'customLabel1': custom_label1,
        'customLabel2': custom_label2,
      }

    self.update_gmc_custom_label(merchant_id, custom_labels)

  def update_gmc_custom_label(self, merchant_id, custom_labels):
    if merchant_id not in self.merchant_ids:
      return

    gmc_store_id = self.merchant_ids[merchant_id]
    items = list(custom_labels.values())
    n = 250
    chunks = [items[x:x + n] for x in range(0, len(items), n)]
    for chunk in chunks:
      gs = self.gss[gmc_store_id]
      result = gs.update_products(chunk)

      logger.info(
        '[GoogleMerchantCustomLabelUpdate] Succeed: %s, Failed: %s',
        result['succeed_cnt'], result['failed_cnt'])
      logger.info(
        '[GoogleMerchantCustomLabelUpdate] %s', chunk)
